@@ -1,6 +1,6 @@
 /*
  * Normaliz
- * Copyright (C) 2007-2022  W. Bruns, B. Ichim, Ch. Soeger, U. v. d. Ohe
+ * Copyright (C) 2007-2025  W. Bruns, B. Ichim, Ch. Soeger, U. v. d. Ohe
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation, either version 3 of the License, or
@@ -137,6 +137,8 @@ ConeProperties all_options() {
     ret.set(ConeProperty::PrimalMode);
     ret.set(ConeProperty::KeepOrder);
     ret.set(ConeProperty::HSOP);
+    ret.set(ConeProperty::OnlyCyclotomicHilbSer);
+    ret.set(ConeProperty::NoQuasiPolynomial);
     ret.set(ConeProperty::Symmetrize);
     ret.set(ConeProperty::NoSymmetrization);
     ret.set(ConeProperty::BigInt);
@@ -156,6 +158,7 @@ ConeProperties all_options() {
     ret.set(ConeProperty::TestLinearAlgebraGMP);
     ret.set(ConeProperty::TestSimplexParallel);
     ret.set(ConeProperty::TestLibNormaliz);
+    ret.set(ConeProperty::NoEmptyOutput);
     ret.set(ConeProperty::NoDescent);
     ret.set(ConeProperty::Descent);
     ret.set(ConeProperty::NoGradingDenom);
@@ -185,6 +188,7 @@ ConeProperties all_options() {
     ret.set(ConeProperty::MinimizePolyEquations);
     ret.set(ConeProperty::DistributedComp);
     ret.set(ConeProperty::UseModularGrading);
+    ret.set(ConeProperty::ExploitAutomsVectors);
     return ret;
 }
 
@@ -283,6 +287,7 @@ void ConeProperties::check_monoid_goals() const{
     ConeProperties copy(*this);
     copy = copy.goals();
     copy.reset(ConeProperty::HilbertBasis);
+    // copy.reset(ConeProperty::ExtremeRays);
     copy.reset(ConeProperty::IsIntegrallyClosed);
     copy.reset(ConeProperty::IsSerreR1);
     copy.reset(ConeProperty::Multiplicity);
@@ -321,6 +326,39 @@ void ConeProperties::check_lattice_ideal_goals() const{
     }
 }
 
+void ConeProperties::preconditions_and_check_series_goals(){
+
+    // OnlyCyclotomicHilbSer ==> NoQuasiPolynomial
+    if(CPs.test(ConeProperty::OnlyCyclotomicHilbSer))
+        CPs.set(ConeProperty::NoQuasiPolynomial);
+
+    // HilbertQuasiPolynomial ==> HilbertSeries
+    if (CPs.test(ConeProperty::HilbertQuasiPolynomial))
+        CPs.set(ConeProperty::HilbertSeries);
+
+    // EhrhartQuasiPolynomial ==> EhrhartSeries
+    if (CPs.test(ConeProperty::EhrhartQuasiPolynomial))
+        CPs.set(ConeProperty::EhrhartSeries);
+
+    // WeightedEhrhartQuasiPolynomial ==> WeightedEhrhartSeries
+    if (CPs.test(ConeProperty::WeightedEhrhartQuasiPolynomial))
+        CPs.set(ConeProperty::WeightedEhrhartSeries);
+
+    if (CPs.test(ConeProperty::WeightedEhrhartSeries)) {
+        CPs.set(ConeProperty::BasicStanleyDec);
+    }
+
+    if(CPs.test(ConeProperty::OnlyCyclotomicHilbSer) && CPs.test(ConeProperty::HSOP))
+        throw BadInputException("HSOP not allowed with OnlyCyclotomicHilbSer");
+
+    if( CPs.test(ConeProperty::NoQuasiPolynomial) && (
+        CPs.test(ConeProperty::HilbertQuasiPolynomial) ||
+        CPs.test(ConeProperty::EhrhartQuasiPolynomial) ||
+        CPs.test(ConeProperty::WeightedEhrhartQuasiPolynomial) )
+    )
+        throw BadInputException("Computation of quasipoloynomial excluded by other required variant.");
+}
+
 void ConeProperties::check_fusion_ring_props() const{
     ConeProperties copy(*this);
     copy.reset(ConeProperty::BigInt);
@@ -344,6 +382,7 @@ void ConeProperties::check_fusion_ring_props() const{
     copy.reset(ConeProperty::MinimizePolyEquations);
     copy.reset(ConeProperty::ModularGradings);
     copy.reset(ConeProperty::UseModularGrading);
+    copy.reset(ConeProperty::NoEmptyOutput);
 
 
     if (copy.any()) {
@@ -446,6 +485,20 @@ void ConeProperties::set_fusion_partition_default() {
 /* add preconditions */
 void ConeProperties::set_preconditions(bool inhomogeneous, bool numberfield) {
 
+    preconditions_and_check_series_goals();
+
+    // homogeneous && EhrhartSeies ==> HilbertSeries
+    if (CPs.test(ConeProperty::EhrhartSeries) && !inhomogeneous) {
+        CPs.set(ConeProperty::HilbertSeries);
+        CPs.set(ConeProperty::NoGradingDenom);
+        CPs.reset(ConeProperty::EhrhartSeries);
+    }
+
+    if(CPs.test(ConeProperty::NoEmptyOutput) && CPs.test(ConeProperty::FusionRings)){
+        CPs.reset(ConeProperty::FusionRings);
+        CPs.set(ConeProperty::SingleLatticePoint);
+    }
+
     if(CPs.test(ConeProperty::NonsimpleFusionRings)){
         CPs.set(ConeProperty::FusionRings);
         CPs.reset(ConeProperty::NonsimpleFusionRings);
@@ -475,11 +528,6 @@ void ConeProperties::set_preconditions(bool inhomogeneous, bool numberfield) {
     if(CPs.test(ConeProperty::MaxDegRepresentations))
         CPs.set(ConeProperty::Representations);
 
-    if (CPs.test(ConeProperty::ExploitAutomsVectors)) {
-        errorOutput() << *this << endl;
-        throw BadInputException("At least one of the listed computation goals not yet implemernted");
-    }
-
     if(CPs.test(ConeProperty::SingleLatticePoint))
         CPs.set(ConeProperty::NoGradingDenom);
 
@@ -509,21 +557,6 @@ void ConeProperties::set_preconditions(bool inhomogeneous, bool numberfield) {
         CPs.reset(ConeProperty::RenfVolume);
     }
 
-    // HilbertQuasipolynomial ==> HilbertSeries
-    if (CPs.test(ConeProperty::HilbertQuasiPolynomial))
-        CPs.set(ConeProperty::HilbertSeries);
-
-    // EhrhartQuasiPolynomial ==> EhrhartSeries
-    if (CPs.test(ConeProperty::EhrhartQuasiPolynomial))
-        CPs.set(ConeProperty::EhrhartSeries);
-
-    // homogeneous && EhrhartSeies ==> HilbertSeries
-    if (CPs.test(ConeProperty::EhrhartSeries) && !inhomogeneous) {
-        CPs.set(ConeProperty::HilbertSeries);
-        CPs.set(ConeProperty::NoGradingDenom);
-        CPs.reset(ConeProperty::EhrhartSeries);
-    }
-
     // EuclideanVolume ==> Volume
     if (CPs.test(ConeProperty::EuclideanVolume))
         CPs.set(ConeProperty::Volume);
@@ -641,11 +674,6 @@ void ConeProperties::set_preconditions(bool inhomogeneous, bool numberfield) {
     if (CPs.test(ConeProperty::ExtremeRays))
         CPs.set(ConeProperty::SupportHyperplanes);
 
-    /*if (CPs.test(ConeProperty::HSOP) && !inhomogeneous){
-        CPs.set(ConeProperty::SupportHyperplanes);
-        CPs.set(ConeProperty::HilbertSeries);
-    }*/
-
     // ModuleGeneratorsOverOriginalMonoid ==> HilbertBasis
     if (CPs.test(ConeProperty::ModuleGeneratorsOverOriginalMonoid))
         CPs.set(ConeProperty::HilbertBasis);
@@ -690,24 +718,10 @@ void ConeProperties::set_preconditions(bool inhomogeneous, bool numberfield) {
     if (CPs.test(ConeProperty::Rank))
         CPs.set(ConeProperty::Sublattice);
 
-    /* if(CPs.test(ConeProperty::Multiplicity) || CPs.test(ConeProperty::HilbertSeries))
-        CPs.set(ConeProperty::SupportHyperplanes);  // to meke them computed if Symmetrize is used
-    */
-
     // we want an ordinary triangulation if one is asked for
     if (CPs.test(ConeProperty::BasicTriangulation) && !numberfield)
         CPs.set(ConeProperty::NoSubdivision);
 
-    // WeightedEhrhartQuasiPolynomial ==> WeightedEhrhartSeries
-    if (CPs.test(ConeProperty::WeightedEhrhartQuasiPolynomial))
-        CPs.set(ConeProperty::WeightedEhrhartSeries);
-
-    // WeightedEhrhart ==> StanleyDec
-    if (CPs.test(ConeProperty::WeightedEhrhartSeries)) {
-        // CPs.set(ConeProperty::Multiplicity);
-        CPs.set(ConeProperty::BasicStanleyDec);
-    }
-
     // Volume + Integral ==> NoGradingDenom
     if (CPs.test(ConeProperty::Volume) || CPs.test(ConeProperty::Integral)) {
         CPs.set(ConeProperty::NoGradingDenom);
@@ -1116,6 +1130,8 @@ vector<string> initializeCPN() {
     CPN.at(ConeProperty::ExploitIsosMult) = "ExploitIsosMult";
     CPN.at(ConeProperty::StrictIsoTypeCheck) = "StrictIsoTypeCheck";
     CPN.at(ConeProperty::HSOP) = "HSOP";
+    CPN.at(ConeProperty::OnlyCyclotomicHilbSer) = "OnlyCyclotomicHilbSer";
+    CPN.at(ConeProperty::NoQuasiPolynomial) = "NoQuasiPolynomial";
     CPN.at(ConeProperty::NoBottomDec) = "NoBottomDec";
     CPN.at(ConeProperty::PrimalMode) = "PrimalMode";
     CPN.at(ConeProperty::Symmetrize) = "Symmetrize";
@@ -1163,6 +1179,7 @@ vector<string> initializeCPN() {
     CPN.at(ConeProperty::TestLinearAlgebraGMP) = "TestLinearAlgebraGMP";
     CPN.at(ConeProperty::TestSimplexParallel) = "TestSimplexParallel";
     CPN.at(ConeProperty::TestLibNormaliz) = "TestLibNormaliz";
+    CPN.at(ConeProperty::NoEmptyOutput) = "NoEmptyOutput";
     CPN.at(ConeProperty::Descent) = "Descent";
     CPN.at(ConeProperty::NoDescent) = "NoDescent";
     CPN.at(ConeProperty::NoGradingDenom) = "NoGradingDenom";
@@ -1215,7 +1232,7 @@ vector<string> initializeCPN() {
     CPN.at(ConeProperty::UseModularGrading) = "UseModularGrading";
 
     // detect changes in size of Enum, to remember to update CPN!
-    static_assert(ConeProperty::EnumSize == 166,"ConeProperties Enum size does not fit! Update cone_property.cpp!");
+    static_assert(ConeProperty::EnumSize == 169,"ConeProperties Enum size does not fit! Update cone_property.cpp!");
     // assert all fields contain an non-empty string
     for (size_t i = 0; i < ConeProperty::EnumSize; i++) {
         // bstd::cout << "iii " << i << "  " << CPN.at(i) << endl;
