@@ -1,6 +1,6 @@
 /*
  * Normaliz
- * Copyright (C) 2007-2022  W. Bruns, B. Ichim, Ch. Soeger, U. v. d. Ohe
+ * Copyright (C) 2007-2025  W. Bruns, B. Ichim, Ch. Soeger, U. v. d. Ohe
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation, either version 3 of the License, or
@@ -61,10 +61,11 @@ volatile sig_atomic_t nmz_interrupted = 0;
 const int default_thread_limit = 8;
 int thread_limit = default_thread_limit;
 bool parallelization_set = false;
-bool no_output_on_interrupt = false;
+bool output_on_interrupt = false;
 bool no_coord_transf = false;
 bool write_lp_file = false;
 bool save_local_solutions = false;
+bool no_empty_output= false;
 
 // bool test_arithmetic_overflow = false;
 // long overflow_test_modulus = 15401;
