@@ -26,6 +26,7 @@
 
 #include "libnormaliz/cone.h"
 #include "libnormaliz/induction.h"
+#include "libnormaliz/normaliz_exception.h"
 
 namespace libnormaliz{
 using std::vector;
@@ -70,6 +71,8 @@ Induction<Integer>::Induction(const vector<Integer>& fus_type, const vector<key_
     /* for(auto& T: Tables)
         T.debug_print();*/
 
+    test_commutativity();
+
     FPdim = 0;
     for(auto& f: fus_type)
         FPdim += f*f;
@@ -86,6 +89,7 @@ Induction<Integer>::Induction(const vector<Integer>& fus_type, const vector<key_
     /* for(auto& T: Tables)
         T.debug_print('$'); */
 
+    // For the computation of eigenvalues and multiplicities
     EVMat.resize(fusion_rank, fusion_rank);
     for(size_t s =0; s < fusion_rank; ++s){
         for(size_t l = 0; l < fusion_rank; ++l){
@@ -101,26 +105,20 @@ Induction<Integer>::Induction(const vector<Integer>& fus_type, const vector<key_
     }
     // EVMat.debug_print();
 
-    MultEV.resize(divisors.size());
-
-    size_t sum_mult = 0;
-    if(verbose)
-        verboseOutput() << "eigenvalues and their multiplicities" << endl;
-    for(size_t i = 0; i< divisors.size(); ++i){
-        MultEV[i] = EVMat.mult_of_eigenvalue(divisors[i]);
-        if(MultEV[i] > 0 && verbose)
-            verboseOutput() << divisors[i] << " mult " <<  MultEV[i] << endl;
-        sum_mult += MultEV[i];
+    if(commutative){
+        eigenvalues_and_mult_commutative();
+    }
+    else{
+        eigenvalues_and_mult_noncommutative();
     }
 
-    if(sum_mult < fusion_rank){
-        if(verbose)
-            verboseOutput() << "Sum of multiplicities of eigenvalues dividing FPdim < fusion_rank" << endl;
-        mult_of_ev_ok = false;
-        return;
-    }
+    Bounds.resize(fusion_rank, fusion_rank);
 
-   Bounds.resize(fusion_rank, fusion_rank);
+   /* for(size_t s = 0; s < fusion_rank; ++s){
+       for(size_t t = 0; t < fusion_rank; ++t)
+           cout << s << " " << t << " " << N(t,0,s) << " " << endl;
+
+   } */
 
     for(size_t j = 0; j< fusion_rank; ++j){
         for(size_t k = 0; k < fusion_rank; ++k){
@@ -145,7 +143,6 @@ Induction<Integer>::Induction(const vector<Integer>& fus_type, const vector<key_
 
         if(t == FPdim)
             continue;
-        LowRepresentations[t].resize(0, fusion_rank);
         Matrix<Integer> InhomEqu(1,fusion_rank +1);
         for(size_t i = 0; i < fusion_rank; ++i)
             InhomEqu[0][i] = fus_type[i];
@@ -158,7 +155,7 @@ Induction<Integer>::Induction(const vector<Integer>& fus_type, const vector<key_
                               Type::inhom_inequalities, NeutralInEqu);
         RepCone.setVerbose(false);
         Matrix<Integer> Reps = RepCone.getLatticePointsMatrix();
-        size_t count_low = 0, count_high = 0;
+        size_t count_high = 0;
         for(size_t i = 0; i < Reps.nr_of_rows(); ++i){
             bool too_large = false;
             for(size_t j = 0; j < fusion_rank; ++j){
@@ -176,10 +173,6 @@ Induction<Integer>::Induction(const vector<Integer>& fus_type, const vector<key_
             vector<Integer> new_rep = Reps[i];
             new_rep.resize(new_rep.size() - 1);
 
-            if(Reps[i][0] == 1){
-                LowRepresentations[t].append(new_rep);
-                count_low++;
-            }
             if(Reps[i][0] == 0){
                 HighRepresentations.append(new_rep);
                 count_high++;
@@ -187,7 +180,7 @@ Induction<Integer>::Induction(const vector<Integer>& fus_type, const vector<key_
 
         }
         if(verbose)
-            verboseOutput() << "divisor " << t << " has " << count_low <<" low reps and " << count_high << " high" << endl;
+            verboseOutput() << "divisor " << t << " has " << count_high << " high" << " representations" << endl;
 
        // LowRepresentations[t].debug_print('$');
     }
@@ -203,6 +196,121 @@ Induction<Integer>::Induction(const vector<Integer>& fus_type, const vector<key_
 
 }
 
+template<typename Integer>
+void Induction<Integer>::test_commutativity(){
+
+    for(size_t i = 0; i < Tables.size(); ++i){
+            for(size_t j = i + 1; j < Tables.size(); ++j){
+                Matrix<Integer> Prod_1 = Tables[i].multiplication(Tables[j]);
+                Matrix<Integer> Prod_2 = Tables[j].multiplication(Tables[i]);
+                if(!Prod_1.equal(Prod_2)){
+                    commutative = false;
+                    return;
+                }
+            }
+    }
+    commutative =  true;
+}
+
+template<typename Integer>
+void Induction<Integer>::eigenvalues_and_mult_commutative(){
+
+    nr_rows_low_part = fusion_rank;
+
+    size_t sum_mult = 0;
+    if(verbose)
+        verboseOutput() << "eigenvalues and their multiplicities in the commutative case" << endl;
+    for(size_t i = 0; i< divisors.size(); ++i){
+        size_t mult_ev = EVMat.mult_of_eigenvalue(divisors[i]);
+        sum_mult += mult_ev;
+        if(mult_ev > 0){
+            EV_mult_n_i[divisors[i]] = make_pair(mult_ev,1);
+            if(verbose){
+                verboseOutput() << divisors[i] << " mult " <<  EV_mult_n_i[divisors[i]].first << endl;
+            }
+        }
+    }
+    if(sum_mult < fusion_rank){
+        if(verbose)
+            verboseOutput() << "Sum of multiplicities of eigenvalues dividing FPdim < fusion_rank" << endl;
+        mult_of_ev_ok = false;
+    }
+}
+
+
+template<typename Integer>
+void Induction<Integer>::eigenvalues_and_mult_noncommutative(){
+
+    if(fusion_rank > 8){
+        throw BadInputException("Fusion rank must be <= 8 for induction matrices in the noncommutative case!");
+    }
+
+    nr_rows_low_part = fusion_rank -3;
+
+    // First we must compute the eigenvalues and multiplicities .
+    // Later on they can be distributed in pairs (d_i, n_i)
+    // where the eigenvalue is n_id_i and the problem that we may havre
+    // n_id_i = n_jd_j for i != j.
+    map<Integer,size_t> MultEV_raw;
+    set<Integer> already_tested;
+    size_t sum_mult = 0;
+
+    for(int i = 1; i <= fusion_rank; ++i){
+        for(size_t j = 0; j < divisors.size(); ++j){
+            Integer ev_cand = i*divisors[j];
+            if(already_tested.find(ev_cand) != already_tested.end())
+                continue;
+            size_t mult_ev = EVMat.mult_of_eigenvalue(ev_cand);
+            if(mult_ev > 0)
+            sum_mult+= mult_ev;
+            if(mult_ev > 0){
+                 MultEV_raw[ev_cand] = mult_ev;
+            }
+            already_tested.insert(ev_cand);
+        }
+    }
+
+    if(verbose){
+        verboseOutput() <<  "Eigenvalues and their multiplicities" << endl;
+        for(auto& mult_this: MultEV_raw){
+            verboseOutput() <<  mult_this.first << " multiplicity " << mult_this.second << endl;
+        }
+    }
+
+    if(sum_mult < fusion_rank){
+        if(verbose)
+            verboseOutput() << "Sum of multiplicities of eigenvalues dividing FPdim < fusion_rank" << endl;
+        mult_of_ev_ok = false;
+        return;
+    }
+
+    // now we want to identify the pairs (n_i, d_i) from their products n_id=i = n_jd_j
+    // we do it only in case rank <= 8 where there is a unique solution: n_i = 2 for exactly
+    // one i:
+    // identify n_id_i of multiplicity >= 4
+    for(auto& ev_x_mult: MultEV_raw){
+        if(ev_x_mult.second < 4){
+            EV_mult_n_i[ev_x_mult.first] = make_pair(ev_x_mult.second,1);
+        }
+        if(ev_x_mult.second >= 4){
+            Integer codeg = ev_x_mult.first/2;
+            EV_mult_n_i[codeg] =make_pair(1, 2);
+
+            size_t k = ev_x_mult.second - 4;
+            if(k > 0)
+                EV_mult_n_i[ev_x_mult.first] = make_pair(k,1);
+        }
+    }
+
+    if(verbose){
+        verboseOutput() << "codegrees with dim of iorreducibles and multiplicities" << endl;
+        for(auto& codeg_mult: EV_mult_n_i){
+            verboseOutput() << codeg_mult.first << " mult " << codeg_mult.second.first << " dim irred " << codeg_mult.second.second << endl;
+        }
+    }
+
+}
+
 template<>
 Induction<renf_elem_class>::Induction(const vector<renf_elem_class>& fus_type, const vector<key_t>& fus_duality , const vector<renf_elem_class>& FusRing, bool verbose){
 
@@ -215,35 +323,94 @@ Integer Induction<Integer>::N(const key_t i, const key_t j, const key_t k){
 }
 
 template<typename Integer>
-void Induction<Integer>::start_low_parts(){
+void Induction<Integer>::solve_system_low_parts(){
+
+    // foirst row and first column are fixed
+    Matrix<Integer> our_equs(0,(nr_rows_low_part -1)*(fusion_rank-1) + 1); // +1 for rhs
+
+    // make row equations
+    for(size_t i = 0; i < nr_rows_low_part -1; ++i){
+        vector<Integer> this_equ((nr_rows_low_part -1)*(fusion_rank-1));
+        for(size_t j = 0; j < fusion_rank -1; ++j){
+            this_equ[i*(fusion_rank -1) +j] = fusion_type[j+1];
+        }
+        // note: entry in first column is n_i
+        this_equ.push_back(-low_m[i+1].first +low_m[i+1].second);
+        our_equs.append(this_equ);
+    }
+
+    // make column equations
+    for(size_t j = 0; j < fusion_rank - 1; ++j){
+        vector<Integer> this_equ((nr_rows_low_part -1)*(fusion_rank-1));
+        for(size_t i = 0; i < nr_rows_low_part - 1; ++i){
+            this_equ[j + i*(fusion_rank -1)] = low_m[i+1].second;  // ?????????
+        }
+        this_equ.push_back(- Bounds[0][j + 1]); // right hand side
+        our_equs.append(this_equ);
+    }
+
+    // cout << "RRR " << our_equs.nr_of_rows() << endl;
+    // cout << "CCC " << our_equs.nr_of_columns() << endl;
+
+    // our_equs.debug_print('E');
+
+    Cone<Integer> LP(Type::inhom_equations, our_equs);
+    LP.setVerbose(false);
+    Matrix<Integer> LowPartsRaw = LP.getLatticePointsMatrix();
+    // LowPartsRaw.debug_print('#');
+
+    for(size_t iii = 0; iii < LowPartsRaw.nr_of_rows(); iii++){
+        Matrix<Integer> our_low_part(nr_rows_low_part, fusion_rank);
+        for(size_t i = 0; i < our_low_part.nr_of_rows(); ++i){
+            our_low_part[i][0] = low_m[i].second;
+        }
+        for(size_t i = 1; i < our_low_part.nr_of_rows(); ++i){
+            for(size_t j = 1; j < our_low_part.nr_of_columns(); ++j){
+                our_low_part[i][j] = LowPartsRaw[iii][(i-1)*(fusion_rank - 1)+ j-1];
+            }
+        }
+        // our_low_part.debug_print('L');
+        LowParts.push_back(our_low_part);
+    }
+}
+
+template<typename Integer>
+void Induction<Integer>::build_low_parts(){
 
     if(!mult_of_ev_ok)
         return;
 
     if(verbose)
-        verboseOutput() << "Computing low parts (rows 1,...,r before sorting)" << endl;
+        verboseOutput() << "Computing low parts (rows 1,...,s before sorting)" << endl;
 
-    vector<Integer> EV;
-    for(size_t i = 0; i < divisors.size(); ++i){
-        for(size_t j = 0; j < MultEV[i]; ++j)
-            EV.push_back(divisors[i]);
+    // Very first we repeat the data accirding to their multiplicities
+    // in second.first and don't need the multiplicities anymore
+
+    vector<pair<Integer, size_t> > EV_n_i;
+
+    for(auto& EV_mult: EV_mult_n_i){
+        for(size_t j = 0; j < EV_mult.second.first; ++j){
+            EV_n_i.push_back(make_pair(EV_mult.first, EV_mult.second.second));
+            // cout << EV_mult.first << " " << EV_mult.second.second << endl;
+        }
+    }
+
+    // First we want to replace eigenvalues by codegrees
+    // Note: eigenvalue n_if_i by f_i si9nce we want m_i = FPdim/f_i
+
+    // convert the n_i to Integer because they are used as such in the following
+    for(auto& t: EV_n_i){
+        Integer dummy = convertTo<Integer>(static_cast<long long>(t.second));
+        low_m.push_back(make_pair(FPdim  / t.first, dummy));
+
+        // cout << "m " << FPdim  / t.first << " mult "  << dummy << endl;
     }
-    for(auto& t: EV)
-        low_m.push_back(FPdim / t);
     sort(low_m.begin(), low_m.end());
 
-    if(verbose){
-        size_t upperbound = 1;
-        for(size_t i = 0; i < fusion_rank; ++i)
-            upperbound *= LowRepresentations[low_m[i]].nr_of_rows();
-        verboseOutput() << "Number of low parts <= " << upperbound << endl;
-    }
-
-
-    Matrix<Integer> matrix_so_far(0, fusion_rank);
-    Matrix<Integer> bounds_so_far(fusion_rank, fusion_rank);
-    build_low_matrices(matrix_so_far, bounds_so_far);
+    solve_system_low_parts();
 
+    // Low parts may come with permuted rows. We select those with
+    // ordered rows
     vector<Matrix<Integer> >  OrderedLowParts;
     for(auto& M: LowParts){
         /* for(size_t i = 0; i < fusion_rank; ++i)
@@ -252,7 +419,7 @@ void Induction<Integer>::start_low_parts(){
         // M.debug_print('C');
         bool ordered = true;
         for(size_t i = 0; i < M.nr_of_rows() - 1; ++i){
-            if(low_m[i] == low_m[i+1]){
+            if(low_m[i].first == low_m[i+1].first){
                 if(M[i] > M[i+1]){
                     // cout << "ord " << i << " " << low_m[i] << " " << low_m[i+1] << endl;
                     ordered = false;
@@ -268,64 +435,18 @@ void Induction<Integer>::start_low_parts(){
     // cout << "Old " << LowParts.size() << " New " << OrderedLowParts.size() << endl;
     swap(LowParts, OrderedLowParts);
 
-    /* if(verbose)
-        verboseOutput() << "Found " << LowParts.size() << " low parts"  << endl;*/
+    if(verbose)
+        verboseOutput() << "Found " << LowParts.size() << " low parts"  << endl;
 
-    /* for(auto& M: LowParts)
-        M.debug_print('P'); */
+    if(verbose){
+        for(auto& M: LowParts)
+            M.debug_print('P');
+    }
     /* for(auto& M: LowPartsBounds)
         M.debug_print('&'); */
 
 }
 
-
-template<typename Integer>
-void Induction<Integer>::build_low_matrices(Matrix<Integer> matrix_so_far, Matrix<Integer> bounds_so_far){
-
-    key_t step = matrix_so_far.nr_of_rows();
-
-    Integer m_step = low_m[step];
-    for(size_t i = 0; i < LowRepresentations[m_step].nr_of_rows(); ++i){
-
-        INTERRUPT_COMPUTATION_BY_EXCEPTION
-
-        Matrix<Integer> NewMatrix = matrix_so_far;
-
-        vector<Integer> cand_extension = LowRepresentations[m_step][i];
-        bool potential_low_part = true;
-
-        Matrix<Integer> check_bounds = bounds_so_far;
-
-        for(size_t j = 0; j < fusion_rank; ++j){
-            for(size_t k = j; k < fusion_rank; ++k){
-                check_bounds[j][k] +=  cand_extension[j] * cand_extension[k];
-                if(check_bounds[j][k] > Bounds[j][k]){
-                   potential_low_part = false;
-                   break;
-                }
-                if(!potential_low_part)
-                    break;
-            }
-        }
-
-        if(!potential_low_part)
-            continue;
-        NewMatrix.append(cand_extension);
-
-        if(NewMatrix.nr_of_rows() == fusion_rank){
-            /* if(verbose && LowParts.size() % 10000 == 0 && LowParts.size() > 0)
-                verboseOutput() << LowParts.size() << " low parts" << endl;
-            LowParts.push_back(NewMatrix);*/
-
-            from_low_to_full(NewMatrix);
-            continue;
-        }
-
-        build_low_matrices(NewMatrix, check_bounds);
-    }
-}
-
-
 template<typename Integer>
 Matrix<Integer> Induction<Integer>::make_allowed_transpositions(Matrix<Integer> FusionMap){
 
@@ -358,21 +479,24 @@ Matrix<Integer> Induction<Integer>::make_allowed_transpositions(Matrix<Integer>
 }
 
 template<typename Integer>
-void Induction<Integer>::from_low_to_full(const Matrix<Integer>& ThisLowPart){
+void Induction<Integer>::from_low_to_full(){
 
 
     if(!mult_of_ev_ok)
         return;
 
     if(verbose)
-        verboseOutput() << "Extending low part to full induction matrices" << endl;
+        verboseOutput() << "Extending low parts to full induction matrices" << endl;
 
+    //First we compute the contribution of the low part to the conditions for induction matrices
 
-    Integer FPdim_so_far =0;
-    for(size_t j = 0; j < fusion_rank; ++j)
-        FPdim_so_far += low_m[j] * low_m[j];
+    for(auto& ThisLowPart: LowParts){
 
-    Integer MinusOne = -1;
+        Integer FPdim_so_far =0;
+        for(size_t j = 0; j < nr_rows_low_part; ++j)
+            FPdim_so_far += low_m[j].first * low_m[j].first;
+
+        Integer MinusOne = -1;
 
         INTERRUPT_COMPUTATION_BY_EXCEPTION
 
@@ -380,9 +504,11 @@ void Induction<Integer>::from_low_to_full(const Matrix<Integer>& ThisLowPart){
         // LowPartsBounds[iii].debug_print('B');
 
         Matrix<Integer> MinusLowBound(fusion_rank, fusion_rank);
-        for(size_t i = 0; i< fusion_rank; ++i){
+        for(size_t i = 0; i< nr_rows_low_part; ++i){
             vector<Integer> cand_extension = ThisLowPart[i];
+            // cout << "cccc " << cand_extension.size() << endl;
             for(size_t j = 0; j < fusion_rank; ++j){
+                // cout << "j "  << j << endl;
                 for(size_t k = j; k < fusion_rank; ++k){
                     MinusLowBound[j][k] +=  cand_extension[j] * cand_extension[k];
                 }
@@ -392,6 +518,9 @@ void Induction<Integer>::from_low_to_full(const Matrix<Integer>& ThisLowPart){
 
         // Bounds.debug_print();
 
+        // Now the conditions for the high part
+        // Remaining: the difference between whe goal and the contribution of the low part
+
         Matrix<Integer> Remaining = Bounds.add(MinusLowBound);
 
         // Remaining.debug_print('R');
@@ -404,7 +533,7 @@ void Induction<Integer>::from_low_to_full(const Matrix<Integer>& ThisLowPart){
             for(size_t k = j; k < fusion_rank; ++k){
                 vector<Integer> this_equ;
                 for(size_t i = 0; i < HighRepresentations.nr_of_rows(); ++i){
-                        this_equ.push_back(HighRepresentations[i][j] * HighRepresentations[i][k]);
+                    this_equ.push_back(HighRepresentations[i][j] * HighRepresentations[i][k]);
                 }
                 this_equ.push_back(-Remaining[j][k]);
                 // cout << j << " " << k << " " << "ttt " <<  this_equ;
@@ -412,6 +541,8 @@ void Induction<Integer>::from_low_to_full(const Matrix<Integer>& ThisLowPart){
             }
         }
 
+        // now the eruation for the FPdim of the center
+
         vector<Integer> this_equ;
         for(size_t i = 0; i < HighRepresentations.nr_of_rows(); ++i){
             Integer m_new = v_scalar_product(HighRepresentations[i], fusion_type);
@@ -423,7 +554,7 @@ void Induction<Integer>::from_low_to_full(const Matrix<Integer>& ThisLowPart){
 
         // InhomEqu.debug_print('&');
 
-        // we use inequalitiesqualities to avoid coordinate transformation
+        // we use inequalities to avoid coordinate transformation
         Matrix<Integer> Copy = InhomEqu;
         Copy.scalar_multiplication(MinusOne);
         InhomEqu.append(Copy);
@@ -434,6 +565,8 @@ void Induction<Integer>::from_low_to_full(const Matrix<Integer>& ThisLowPart){
         C.compute(ConeProperty::HilbertBasis, ConeProperty::Projection);
         Matrix<Integer> LP = C.getLatticePointsMatrix();
 
+        // LP.debug_print('P');
+
         if(talkative && LP.nr_of_rows() == 0)
             verboseOutput() << "N" << endl;
 
@@ -450,11 +583,13 @@ void Induction<Integer>::from_low_to_full(const Matrix<Integer>& ThisLowPart){
 
             }
             InductionMatrices.push_back(IndMat);
-            if(verbose)
+            /*if(verbose){
                 IndMat.debug_print('I');
-            if(talkative)
-                verboseOutput() << "I" << endl;
+                verboseOutput() << endl;
+            }*/
         }
+
+    }
 }
 
 template<typename Integer>
@@ -514,53 +649,11 @@ void Induction<Integer>::compute(){
 
     if(!mult_of_ev_ok)
         return;
-    start_low_parts();
+    build_low_parts();
+    from_low_to_full();
     augment_induction_matrices();
 }
 
-/* Not used anamore
-template<typename Integer>
-void Induction<Integer>::extend_matrix(Matrix<Integer> matrix_so_far, key_t rep_index,
-                       Matrix<Integer> bounds_so_far, Integer FPdim_so_far){
-
-    if(rep_index >= HighRepresentations.nr_of_rows())
-        return;
-
-    vector<Integer> cand_ext = HighRepresentations[rep_index];
-    Integer new_m = v_scalar_product(cand_ext, fusion_type);
-
-    while(true){
-
-        extend_matrix(matrix_so_far, rep_index +1, bounds_so_far, FPdim_so_far);
-        FPdim_so_far += new_m* new_m;
-        if(FPdim_so_far > FPSquare){
-            return;
-        }
-        bool potential_solution = true;
-        if(FPdim_so_far < FPSquare)
-            potential_solution = false;
-
-        for(size_t j = 0; j < fusion_rank; ++j){
-            for(size_t k = j; k < fusion_rank; ++k){
-                bounds_so_far[j][k] +=  cand_ext[j] * cand_ext[k];
-                if(bounds_so_far[j][k] > Bounds[j][k]){
-                    return;
-                }
-                if(bounds_so_far[j][k] < Bounds[j][k])
-                    potential_solution = false;
-            }
-        }
-        matrix_so_far.append(cand_ext);
-
-        if(potential_solution){
-            InductionMatrices.push_back(matrix_so_far);
-            // matrix_so_far.debug_print('$');
-            return;
-        }
-
-    }
-}
-*/
 
 template class Induction<mpz_class>;
 template class Induction<long long>;
