@@ -1,6 +1,6 @@
 /*
  * Normaliz
- * Copyright (C) 2007-2022  W. Bruns, B. Ichim, Ch. Soeger, U. v. d. Ohe
+ * Copyright (C) 2007-2025  W. Bruns, B. Ichim, Ch. Soeger, U. v. d. Ohe
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation, either version 3 of the License, or
@@ -94,6 +94,10 @@ enum InputType {
     fusion_duality,
     candidate_subring,
     fusion_type_for_partition,
+    fusion_ring_map,
+    fusion_image_type,
+    fusion_image_ring,
+    fusion_image_duality,
     //
     // precomputed data
     //
@@ -153,6 +157,8 @@ enum Param {
     decimal_digits,
     gb_degree_bound,
     gb_min_degree,
+    modular_grading,
+    chosen_fusion_ring,
     not_a_num_param
 };
 }  // end namespace NumParam
@@ -353,6 +359,18 @@ inline InputType to_type(const string& type_string) {
     if (type_string == "candidate_subring") {
         return Type::candidate_subring;
     }
+    if (type_string == "fusion_image_type") {
+        return Type::fusion_image_type;
+    }
+    if (type_string == "fusion_image_duality") {
+        return Type::fusion_image_duality;
+    }
+    if (type_string == "fusion_image_ring") {
+        return Type::fusion_image_ring;
+    }
+    if (type_string == "fusion_ring_map") {
+        return Type::fusion_ring_map;
+    }
 
     throw BadInputException("Unknown type \"" + type_string + "\"!");
     return Type::integral_closure;
@@ -376,7 +394,8 @@ inline bool type_is_vector(InputType type) {
         type == Type::offset || type == Type::open_facets || type == Type::projection_coordinates
         || type == Type::scale || type == Type::rational_offset || type == Type::gb_weight
          || type == Type::fusion_type || type == Type::fusion_duality || type == Type::fusion_type_for_partition
-        || type == Type::candidate_subring) {
+        || type == Type::candidate_subring || type == Type::fusion_image_duality || type == Type::fusion_image_ring
+        || type == Type::fusion_image_type) {
         return true;
     }
     return false;
@@ -421,6 +440,10 @@ inline NumParam::Param to_numpar(const string& type_string) {
         return NumParam::gb_degree_bound;
     if (type_string == "gb_min_degree")
         return NumParam::gb_min_degree;
+    if (type_string == "modular_grading")
+        return NumParam::modular_grading;
+    if (type_string == "chosen_fusion_ring")
+        return NumParam::chosen_fusion_ring;
 
     return NumParam::not_a_num_param;
 }
@@ -444,6 +467,10 @@ inline string numpar_to_string(const NumParam::Param& numpar) {
         return "gb_degree_bound";
     if (numpar == NumParam::gb_min_degree)
         return "gb_min_degree";
+    if (numpar == NumParam::modular_grading)
+        return "modular_grading";
+    if (numpar == NumParam::chosen_fusion_ring)
+        return "chosen_fusion_ring";
     if (numpar == NumParam::not_a_num_param)
         return "not_a_num_param";
     assert(false);
