@@ -1,6 +1,6 @@
 /*
  * Normaliz
- * Copyright (C) 2007-2022  W. Bruns, B. Ichim, Ch. Soeger, U. v. d. Ohe
+ * Copyright (C) 2007-2025  W. Bruns, B. Ichim, Ch. Soeger, U. v. d. Ohe
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation, either version 3 of the License, or
@@ -32,7 +32,7 @@ using std::endl;
 using std::ifstream;
 
 static void printCopying() {
-    cout << "Copyright (C) 2007-2022  The Normaliz Team, University of Osnabrueck." << endl
+    cout << "Copyright (C) 2007-2025  The Normaliz Team, University of Osnabrueck." << endl
          << "This program comes with ABSOLUTELY NO WARRANTY; This is free software," << endl
          << "and you are welcome to redistribute it under certain conditions;" << endl
          << "See COPYING for details." << endl;
@@ -437,8 +437,8 @@ bool OptionsHandler::handle_options(vector<string>& LongOptions, string& ShortOp
             no_matrices_output = true;
             continue;
         }
-        if (LongOption == "NoOutputOnInterrupt") {
-            no_output_on_interrupt = true;
+        if (LongOption == "OutputOnInterrupt" || LongOption == "OOI") {
+            output_on_interrupt = true;
             continue;
         }
         if (LongOption == "NoHilbertBasisOutput") {
@@ -457,6 +457,11 @@ bool OptionsHandler::handle_options(vector<string>& LongOptions, string& ShortOp
             write_all_files = true;
             continue;
         }
+        if(LongOption == "NoEmptyOutput"){
+            to_compute.set(ConeProperty::NoEmptyOutput);
+            no_empty_output = true;
+            continue;
+        }
         if(LongOption == "UWP"){
             to_compute.set(ConeProperty::UseWeightsPatching);
             continue;
@@ -472,7 +477,6 @@ bool OptionsHandler::handle_options(vector<string>& LongOptions, string& ShortOp
         if(LongOption == "DCM"){
             to_compute.set(ConeProperty::DistributedComp);
             continue;
-
         }
         if(LongOption == "NHM"){
             to_compute.set(ConeProperty::NoHeuristicMinimization);
